import pydantic

class CompressorAbout(pydantic.BaseModel):
  """
  Pydantic model that defines the metadata for a compressor.
  """
  name: str
  description: str
  version: str
  author: str = None
  license: str = None
  url: str = None


# __package__ = "litl.compressors"

from ..datawrapper import DataWrapper
from ..blobs import Blob
from .compressor_about import CompressorAbout

class Compressor():
  """
  Interface for all compressors
  """

  @classmethod
  def about(cls) -> CompressorAbout:
    """
    Returns metadata about the compressor, including its name, description, version, author, license, and URL
    """
    raise NotImplementedError()
  
  @classmethod
  def optuna_suggest_config(cls, trial):
    """
    Optional method that can be used to suggest hyperparameters for the compressor using Optuna
    """
    raise NotImplementedError()
  

  @classmethod
  def blob_class(cls) -> type[Blob]:
    """
    Returns the type of Blob object that the Compressor uses to store compressed data
    """
    raise NotImplementedError()

  @classmethod
  def compress(cls, data: DataWrapper, config: dict) -> tuple[Blob, dict]:
    """
    Compresses the data in the given DataWrapper object

    Args:
      data (DataWrapper): The data to be compressed
      config (pydantic.BaseModel): The configuration for the compressor

    Returns:
      tuple[Blob, pydantic.BaseModel]: A tuple containing the compressed data in a Blob object and the configuration used for compression
    """
    raise NotImplementedError()

  @classmethod
  def decompress(cls, data: Blob, meta: dict) -> DataWrapper:
    """
    Decompresses the data in the given Blob object

    Args:
      data (Blob): The compressed data to be decompressed, guaranteed to be the same type as the one returned by compress
      meta (dict): The metadata associated with the compressed data, guaranteed to be the same as the one used during compression

    Returns:
      DataWrapper: The decompressed data
    """
    raise NotImplementedError()




import os
import numpy as np
import torch

import io

from . import Blob

class ArrayBlob(Blob):
  def __init__(self, data: np.ndarray) -> None:
    if isinstance(data, np.ndarray):
      self.array = data
    else:
      raise ValueError(f'Unsupported data type {type(data)}. Expected numpy.ndarray')
    
  @classmethod
  def from_bytes(cls, data: bytes) -> 'ArrayBlob':
    """
    Create a new ArrayBlob instance from bytes
    """
    # load the numpy array from bytes, keep the original dtype and shape

    return cls(np.load(io.BytesIO(data), allow_pickle=True))
  
  def to_bytes(self) -> bytes:
    """
    Convert the ArrayBlob to bytes
    """
    with io.BytesIO() as buffer:
      # Save the numpy array to bytes
      np.save(buffer, self.array, allow_pickle=True)
      data = buffer.getvalue()

    return data


import os
from . import Blob

class BytesBlob(Blob):
  def __init__(self, data: bytes) -> None:
    """
    Initialize a BytesBlob with the given data
    
    Args:
      data (bytes): The data to be stored in the blob
    """
    self.data = data

  @classmethod
  def from_bytes(cls, data: bytes) -> 'BytesBlob':
    """
    Create a new BytesBlob instance from bytes
    
    Args:
      data (bytes): The data to be stored in the blob
    
    Returns:
      BytesBlob: A new instance of BytesBlob containing the data
    """
    return cls(data)
  
  def to_bytes(self) -> bytes:
    """
    Convert the BytesBlob to bytes
    
    Returns:
      bytes: The data stored in the blob
    """
    return self.data



import os

import torch
import numpy as np
import torcheval.metrics as metrics
# from torchmetrics.image import StructuralSimilarityIndexMeasure
from pytorch_msssim import ssim

from .datawrapper import DataWrapper
from .blobs import Blob

class Evaluator():

  def quality(original: DataWrapper, reconstructed: DataWrapper) -> dict:
    """
    Returns the quality metrics of the reconstructed data compared to the original data
    """
    original_data = original.tensor()
    reconstructed_data = reconstructed.tensor()

    mse = metrics.MeanSquaredError()
    psnr = metrics.PeakSignalNoiseRatio(data_range=original_data.max().item())

    mse_metric = mse.update(reconstructed_data.flatten(), original_data.flatten()).compute().item()
    del mse

    psnr_metric = psnr.update(reconstructed_data.flatten(), original_data.flatten()).compute().item()
    del psnr

    ssim_metric = ssim(reconstructed_data, original_data, data_range=original_data.max().item()).item()

    l1_metric = torch.mean(torch.abs(reconstructed_data - original_data)).item()

    return {
      'mse': mse_metric,
      'psnr': psnr_metric,
      'ssim': ssim_metric,
      'l1': l1_metric,
    }

  def size(compressed_bytes: bytes, original_size: int) -> int:
    """
    Returns the size metrics of the blob
    """
    blob_size = len(compressed_bytes)

    return {
      'original_size': original_size,
      'compressed_size': blob_size,
      'compression_ratio': original_size / blob_size,
    }



import argparse
import os
from functools import partial
import time

import numpy as np
import torch

from .compressors import Compressor
from .datawrapper import DataWrapper
from .blobs import Blob
from .evaluator import Evaluator
from .dot_litl import DotLitl

import importlib

def _discover_compressors():
    """
    Discover all compressors in the entry points
    """
    return {
        ep.name: ep.load()
        for ep in importlib.metadata.entry_points(group="litl.compressors")
    }

global _AVAILABLE_COMPRESSORS
_AVAILABLE_COMPRESSORS = None

def get_compressor(name: str) -> Compressor:
    """
    Get a compressor by name
    """
    global _AVAILABLE_COMPRESSORS
    if not _AVAILABLE_COMPRESSORS:
        _AVAILABLE_COMPRESSORS = _discover_compressors()

    if name in _AVAILABLE_COMPRESSORS:
        # if the compressor is in the entry points, return it
        return _AVAILABLE_COMPRESSORS[name]
    elif ":" in name:
        # if the compressor is not in the entry points, try to load it from a file
        try:
            path, classname = name.rsplit(':', 1)
            spec = importlib.util.spec_from_file_location(path, os.path.expanduser(path))
            mod  = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(mod)
        except ValueError as e:
          raise ImportError(f"Failed to load compressor script file at path {path}, possibly it doesn't exist") from e
          
        try:
          cls = getattr(mod, classname)
        except AttributeError:
            raise ValueError(f"Compressor {name} does not have class {classname}")
        
        if not issubclass(cls, Compressor):
            raise ValueError(f"Compressor {name} is not a subclass of Compressor")
        return cls
    
    else:
        raise ValueError(f"Compressor {name} not found in entry points. Available compressors in pip: {list(_AVAILABLE_COMPRESSORS.keys())} or please provide a path to a compressor file in the format 'path/to/file.py:CompressorClassName'")
        

def compress(compressor_name: str, data_path: str, config: dict, litl_file_path: str=None) -> tuple[Blob, dict, dict]:
    """
    Compress data using the specified compressor
    Args:
        compressor_name (str): The name of the compressor to use
        data_path (str): The path to the data file to compress
        config (dict): Configuration for the compressor
        litl_file_path (str, optional): If provided, save the compressed data to a .litl file
        
    Returns:
        tuple: A tuple containing the compressed blob, metadata, and metrics
    """
    # load data
    compressor_class = get_compressor(compressor_name)

    data = DataWrapper.from_file(data_path)
    original_size = os.path.getsize(data_path)

    start_time = time.time()
    compressed_blob, meta = compressor_class.compress(data, config)
    compression_time = time.time() - start_time

    del data

    start_time = time.time()
    compressed_bytes = compressed_blob.to_bytes()
    io_time = time.time() - start_time
    
    metrics = Evaluator.size(compressed_bytes, original_size)

    metrics.update({
        'compression_time': compression_time,
        'io_time': io_time,
    })

    if litl_file_path is not None:
        # save to litl file
        total_size = DotLitl.save(litl_file_path, compressed_bytes, meta, compressor_name, compressor_class.about().version)
        metrics.update({
            'total_litl_size': total_size,

        })

    return compressed_blob, meta, metrics

def decompress(compressor_name=None, original_data_path: str=None, compressed_bytes: bytes=None, meta: dict=None, litl_file_path: str=None, decompressed_path: str=None) -> tuple[DataWrapper, dict]:
    """
    Decompress data using the specified compressor
    Args:
        compressor_name (str): The name of the compressor to use
        original_data_path (str): The path to the original data file for quality evaluation
        compressed_bytes (bytes): The compressed data bytes
        meta (dict): Metadata for the compressed data
        
        litl_file_path (str): Provide instead of compressor_name, original_data_bytes, and meta to read everything the compressed data from a .litl file
        
        decompressed_path (str, optional): If provided, save the decompressed data to this path
        
    Returns:
        tuple: A tuple containing the decompressed data and metrics
    """

    if litl_file_path is not None:
        # read from litl file
        blob_bytes, compressor_name, compressor_version, meta, litl_version = DotLitl.read(litl_file_path)
        compressor_class = get_compressor(compressor_name)

        if compressor_class.about().version != compressor_version:
            raise ValueError(f"Compressor version mismatch: loaded compressor is v{compressor_class.about().version}, originally compressed with v{compressor_version}")

        compressed_blob_class = compressor_class.blob_class()
        compressed_blob = compressed_blob_class.from_bytes(blob_bytes)
    elif compressed_bytes is not None and meta is not None:
        # read from bytes
        compressor_class = get_compressor(compressor_name)
        compressed_blob_class = compressor_class.blob_class()
        compressed_blob = compressed_blob_class.from_bytes(compressed_bytes)
    else:
        raise ValueError("Either compressed_bytes with meta or litl_file_path must be provided")
    
    start_time = time.time()
    decompressed_data = compressor_class.decompress(compressed_blob, meta)
    decompression_time = time.time() - start_time

    metrics = {
        'decompression_time': decompression_time,
    }

    if original_data_path is not None:
        original_data = DataWrapper.from_file(original_data_path)
    
        metrics.update(Evaluator.quality(original_data, decompressed_data))

    if decompressed_path is not None:
        # save decompressed data
        decompressed_data.save_to_file(decompressed_path)

    return decompressed_data, metrics


def evaluate(compressor_name: str, data_path: str, config, decompressed_path: str=None, litl_file_path: str=None):
    """
    Evaluate the compression and decompression process by compressing and decompressing the data.
    Args:
        compressor_name (str): The name of the compressor to use
        data_path (str): The path to the data file to compress
        config (dict): Configuration for the compressor
        
        decompressed_path (str, optional): If provided, save the decompressed data to this path
        litl_file_path (str, optional): If provided, save the compressed data to a .litl file
        
    Returns:
        tuple: A tuple containing the decompressed data and metrics
    """

    # make deterministic
    torch.manual_seed(0)
    np.random.seed(0)

    # optimize cuda
    torch.set_float32_matmul_precision("medium")

    compressor = get_compressor(compressor_name)

    metrics = {}

    # compress
    compressed_blob, meta, compress_metrics = compress(compressor_name, data_path, config, litl_file_path)
    metrics.update(compress_metrics)

    # decompress
    decompressed_data, decompress_metrics = decompress(compressor_name, data_path, compressed_blob.to_bytes(), meta, decompressed_path=decompressed_path)
    metrics.update(decompress_metrics)

    return decompressed_data, metrics



import argparse
import os
import sys
import json
from importlib import metadata as importlib_metadata

import rich
from rich.console import Console
from rich.progress import Progress

from .run import compress, decompress, evaluate
from .dot_litl import DotLitl

def parse_args():
    parser = argparse.ArgumentParser(description="Neat CLI for lossy compression via litl")
    parser.add_argument("-m", "--metrics_path", help="Where to save the metrics json")
    parser.add_argument("--version", action="version", version=f"litl v{importlib_metadata.version('litl')}", help="Show the version of litl")

    subparsers = parser.add_subparsers(dest="command", required=True)

    compress_parser = subparsers.add_parser("compress", help="Compress data into a .litl file")
    compress_parser.add_argument("compressor", help="Compressor to use")
    compress_parser.add_argument("original_path", help="Where to find input data to compress")
    compress_parser.add_argument("config_path", help="Where to find config json")
    compress_parser.add_argument("compressed_path", help="Where to save the compressed litl file")

    decompress_parser = subparsers.add_parser("decompress", help="Decompress a .litl file")
    decompress_parser.add_argument("compressed_path", help="Where to find the compressed litl file")
    decompress_parser.add_argument("decompressed_path", help="Where to save the decompressed data")

    decompress_parser.add_argument("-o", "--original_path", help="Where to find original data to compare against, optional")

    evaluate_parser = subparsers.add_parser("evaluate", help="Compress and decompress data, while evaluating the compression")
    evaluate_parser.add_argument("compressor", help="Compressor to use")
    evaluate_parser.add_argument("original_path", help="Where to find input data to compress")
    evaluate_parser.add_argument("config_path", help="Where to find config json")

    evaluate_parser.add_argument("-c", "--compressed_path", help="Where to save the intermediate compressed .litl file")
    evaluate_parser.add_argument("-d", "--decompressed_path", help="Where to save the decompressed data")

    info_parser = subparsers.add_parser("info", help="Print metadata stored in the .litl file")
    info_parser.add_argument("input_path", help="Where to find the .litl file")

    return parser.parse_args()

ascii_art = \
"""┓• ┓
┃┓╋┃
┗┗┗┗"""

def main():
    console = Console()
    args = parse_args()
    # fancy rich printing
    console.print(f"[bold green]{ascii_art}[/bold green]")
    console.print(f"[bold]Running litl [green]{args.command}[/green][/bold]")
    
    width = console.width

    try:
        
      if args.command == "compress":
          with open(args.config_path, "r") as f:
              config = f.read()
          config = json.loads(config)
              
          with console.status("[bold green]Compressing...[/bold green]"):
            compressed_blob, meta, metrics = compress(args.compressor, args.original_path, config=config, litl_file_path=args.compressed_path)
            
          console.print(f"Compressed data saved to {args.compressed_path}")
          console.print(f"Metrics: {metrics}")

      elif args.command == "decompress":

          with console.status("[bold green]Decompressing...[/bold green]"):
            decompressed_data, metrics = decompress(litl_file_path=args.compressed_path, decompressed_path=args.decompressed_path, original_data_path=args.original_path)

          console.print(f"Decompressed data saved to {args.decompressed_path}")
          console.print(f"Metrics: {metrics}")

      elif args.command == "evaluate":
          
          with open(args.config_path, "r") as f:
              config = f.read()
          config = json.loads(config)
          
          with console.status("[bold green]Evaluating compression...[/bold green]"):
            decompressed_data, metrics = evaluate(args.compressor, args.original_path, config=config, decompressed_path=args.decompressed_path, litl_file_path=args.compressed_path)
          
          if args.compressed_path:
              console.print(f"Compressed data saved to {args.compressed_path}")

          if args.decompressed_path:
              console.print(f"Decompressed data saved to {args.decompressed_path}")

          console.print("Metrics:", metrics)

      elif args.command == "info":
          
          with console.status("[bold green]Reading metadata...[/bold green]"):
            compressed_blob, compressor_name, compressor_version, meta, litl_version = DotLitl.read(args.input_path)
            
          console.print(f"File {args.input_path} was compressed with {compressor_name} v{compressor_version} using litl v{litl_version}")
          console.print("Metadata:", meta)

      else:
          raise ValueError("Invalid command. Use 'compress', 'decompress', or 'evaluate'.")

      if args.metrics_path is not None:
          with open(args.metrics_path, "w") as f:
              f.write(str(metrics))
          console.print(f"Metrics saved to {args.metrics_path}")
      
    except Exception as e:
        console.print_exception(show_locals=True, width=console.width)
        sys.exit(1)
        
    return 0


import os, struct
import zstd
import io
from importlib import metadata as importlib_metadata
import pydantic
import hashlib
import json
import warnings

class LitlMeta(pydantic.BaseModel):
    litl_version: str
    compressor_id: str
    compressor_version: str
    compressor_meta: dict

    def serialize_value(self) -> str:
        """
        Serialize the metadata to a string
        """
        li = list(self.model_dump().values())
        return json.dumps(li)
    
    @classmethod
    def deserialize_value(cls, value: str) -> 'LitlMeta':
        """
        Deserialize the metadata from a string
        """
        try:
            values_list = json.loads(value)
        except json.JSONDecodeError as e:
            raise ValueError(f"Failed to deserialize .litl metadata: {e}")
        
        if len(values_list) != len(cls.model_fields):
            raise ValueError(f"Failed to deserialize .litl metadata: expected {len(cls.model_fields)} values, got {len(values_list)}")
        
        values = dict(zip(cls.model_fields.keys(), values_list))
        return cls(**values)

class DotLitl:
    """
    A class to save and read .litl files
    """
    version = 1

    @classmethod
    def zstd_compress_bytes(cls, data: bytes) -> bytes:
        """
        Compress the data using Zstandard
        """
        level = 3
        compressed = zstd.compress(data, level)
        return compressed
    
    @classmethod
    def zstd_decompress_bytes(cls, data: bytes) -> bytes:
        """
        Decompress the data using Zstandard
        """
        return zstd.decompress(data)
    

    @classmethod
    def make_meta(cls, meta: dict, compressor_id: str, compressor_version: str) -> str:
        """
        Create a metadata dictionary
        """
        litl_version = importlib_metadata.version("litl")
        
        litl_meta = LitlMeta(
            litl_version=litl_version,
            compressor_id=compressor_id,
            compressor_version=compressor_version,
            compressor_meta=meta
        ).serialize_value()
        return litl_meta
    
    @classmethod
    def checksum(cls, blob_bytes: bytes, meta_bytes: bytes) -> int:
        """
        Calculate a checksum for the data
        """
        hasher = hashlib.sha256(usedforsecurity=False)
        hasher.update(blob_bytes)
        hasher.update(meta_bytes)
        checksum = hasher.digest()[:4]
        return int.from_bytes(checksum, byteorder="little")

    @classmethod
    def save(cls, path: str, blob_bytes: bytes, meta: dict, compressor_name: str, compressor_version: str) -> int:
        """
        Save the blob to a .litl file

        Byte structure is as follows:
        - 4 bytes: magic number (LITL)
        - 1 byte: version
        - 4 bytes: checksum
        - 4 bytes: length of meta
        - meta: metadata (as bytes)
        - blob: blob data (as bytes)
        
        Args:
            path (str): The path to save the .litl file
            blob_bytes (bytes): The compressed blob data
            meta (dict): Metadata to be saved
            compressor_name (str): Name of the compressor used
            compressor_version (str): Version of the compressor used
            
        Returns:
            int: The total size of the saved .litl file
        """
        meta_bytes = cls.zstd_compress_bytes(cls.make_meta(meta, compressor_name, compressor_version).encode('utf-8'))
        blob_bytes = cls.zstd_compress_bytes(blob_bytes)
        checksum = cls.checksum(blob_bytes, meta_bytes)

        with open(path, 'wb') as f:
            f.write(b"LITL")
            f.write(struct.pack("<B", cls.version))
            f.write(struct.pack("<I", checksum))
            f.write(struct.pack("<I", len(meta_bytes)))
            f.write(meta_bytes)
            f.write(blob_bytes)

        total_size = os.path.getsize(path)

        return total_size

    @classmethod
    def read(cls, path: str) -> tuple[bytes, str, str, dict, str]:
        """
        Read the blob and metadata from a .litl file
        """
        try:
            with open(path, 'rb') as f:
                magic = f.read(4)
                if magic != b"LITL":
                    raise ValueError("Invalid file format")

                version = struct.unpack("<B", f.read(1))[0]
                checksum = struct.unpack("<I", f.read(4))[0]

                if version != cls.version:
                    raise ValueError(f"Unsupported version: {version}")
                
                meta_length = struct.unpack("<I", f.read(4))[0]
                # print("Meta length:", meta_length)
                meta_bytes = f.read(meta_length)
                blob_bytes = f.read()

            # checksum validation
            new_checksum = cls.checksum(blob_bytes, meta_bytes)
            if new_checksum != checksum:
                 warnings.warn(
                    f"Checksum mismatch in file {path}: expected {checksum:08x}, "
                    f"got {new_checksum:08x}. File may be corrupted or tampered with.",
                    UserWarning
                )
            
            # decompress meta
            blob_bytes = cls.zstd_decompress_bytes(blob_bytes)
            meta_bytes = cls.zstd_decompress_bytes(meta_bytes)

            meta = LitlMeta.deserialize_value(meta_bytes.decode('utf-8'))

            return blob_bytes, meta.compressor_id, meta.compressor_version, meta.compressor_meta, meta.litl_version
        except IOError as e:
            raise ValueError(f"Failed to read file {path}: {e}")


# __package__ = "litl.datawrapper"

import numpy as np
import torch
import os

class DataWrapper():
  """
  A wrapper class for structured datasets

  Args:
    data (torch.Tensor or np.ndarray): The data to be wrapped. It can be a torch tensor or a numpy array.

  """
  def __init__(self, data: torch.Tensor | np.ndarray):
    if isinstance(data, np.ndarray):
      self.array = data.copy()
    elif isinstance(data, torch.Tensor):
      self.array = data.numpy().copy()
    else:
      raise ValueError(f'data must be a `torch.Tensor` or `numpy.ndarray`, but got {type(data)}')

  @classmethod
  def from_file(cls, path: str):
    """
    Create a new `DataWrapper` instance from a file. The file format can be either .npy, .np, or .pt

    Args:
      path (str): The path to the file to load data from
      
    """
    extension = os.path.splitext(path)[-1][1:]
    if extension == 'npy' or extension == 'np':
      array = np.load(path, allow_pickle=False)
      return cls(array)
    elif extension == 'pt':
      array = torch.load(path, weights_only=False)
      return cls(array)
    else:
      raise ValueError(f'Unsupported file format to load from: .{extension}')

  def save_to_file(self, path: str):
    """
    Save the data to a file. The file format can be either .npy, .np, or .pt

    Args:
      path (str): The path to the file to save data to
    """
    extension = os.path.splitext(path)[-1][1:]
    if extension == 'npy' or extension == 'np':
      np.save(path, self.array, allow_pickle=False)
    elif extension == 'pt':
      torch.save(self.tensor(), path, _use_new_zipfile_serialization=False)
    elif extension == 'nc':
      self.save_netcdf(path)
    else:
      raise ValueError(f'Unsupported file format to save as: ".{extension}"')
    
  def save_netcdf(self, path: str, dimnames: list[str]=None):
    """
    Save the data to a NetCDF file.

    This function requires netCDF4, which is an optional dependency. Please install it with `pip install netCDF4`

    Args:
      path (str): The path to the NetCDF file to save data to

      dimnames (list[str], optional): The names of the dimensions. If not provided, default names will be used. \
      If the dataset has more than 4 dimensions, you must provide the names manually. \
      Note, that if you provide more names than dimensions, the first `len(dimnames) - len(self.array.shape)` names will be ignored.

    """
    # import netCDF4 only if needed
    try:
      from netCDF4 import Dataset
    except ImportError:
      raise ImportError('This function requires netCDF4, which is an optional dependency. Please install it with `pip install netCDF4`')

    # come up with dimnames
    default_dimnames = ['t', 'z', 'y', 'x']
    if dimnames is None and len(self.array.shape) > len(default_dimnames):
      raise ValueError(f'Dataset has more than {len(default_dimnames)} dimensions, please provide their names manually through `dimnames` argument')
    elif dimnames is None:
      dimnames = default_dimnames
    elif len(dimnames) < len(self.array.shape):
      raise ValueError(f'Dataset has {len(self.array.shape)} dimensions, but only {len(dimnames)} names were provided through `dimnames` argument')
      
    lendiff = len(dimnames) - len(self.array.shape)
    with Dataset(path, 'w', format='NETCDF4') as nc:
      # create dimensions
      for i, dim in enumerate(self.array.shape):
        name = dimnames[lendiff + i]
        nc.createDimension(name, dim)

      var = nc.createVariable('data', self.array.dtype, ('dim',))
      var[:] = self.array.flatten()

      var.units = 'unknown'
      var.description = 'Data saved by LITL'

  def tensor(self) -> torch.Tensor:
    """
    Convert the data to a torch tensor
    """
    return torch.from_numpy(self.array)

  def numpy(self) -> np.ndarray:
    return self.array
  
  def dim(self) -> int:
    return len(self.array.shape)
  
  def shape(self) -> tuple:
    return self.array.shape

  


from .blob import Blob
from .array_blob import ArrayBlob
from .bytes_blob import BytesBlob
from .model_blob import ModelBlob

__all__ = [
    "Blob",
    "ArrayBlob",
    "BytesBlob",
    "ModelBlob",
]


class Blob():
  """
  Blob is an abstract class that defines the interface for compressed data

  You don't need to store metadata in the blob, return it from compress method along with blob instead
  """

  def __init__(self, *args, **kwargs) -> None:
    """
    Initialize an empty blob. Please use set_data to set the data
    """
    raise NotImplementedError()

  @classmethod
  def from_bytes(cls, data: bytes) -> None:
    """
    Create a new blob instance from bytes
    """
    raise NotImplementedError()
  
  def to_bytes(self) -> bytes:
    """
    Convert the blob to bytes.
    """
    raise NotImplementedError()


import os
import torch
from . import Blob
import io

class ModelBlob(Blob):
  def __init__(self, model: torch.nn.Module) -> None:
    self.model = model

  @classmethod
  def from_bytes(cls, data: bytes) -> 'ModelBlob':
    """
    Create a new ModelBlob instance from bytes

    Args:
      data (bytes): The bytes to be loaded into the model blob
    """
    # Load the model state dict from the bytes
    buffer = io.BytesIO(data)
    model = torch.load(buffer, map_location='cpu')
    return cls(model)
  
  def to_bytes(self) -> bytes:
    """
    Convert the ModelBlob to bytes
    """
    buffer = io.BytesIO()
    torch.save(self.model, buffer)
    return buffer.getvalue()



import importlib
import os

from .datawrapper import DataWrapper
from .evaluator import Evaluator
from .run import compress, decompress
from .dot_litl import DotLitl

__all__ = [
    "DataWrapper",
    "Evaluator",
    "compress",
    "decompress",
    "DotLitl",
]
